import threading
from io import StringIO

from src.treact.renderers.console import ConsoleRenderer


def inner_fn(context):
    with context.create_node('inner_fn'):
        # print('inside inner_fn')
        context.create_node('inner_fn_child')


def test_multithreading(thread_name):
    # print(thread_name, 'start')

    buf = StringIO()
    with ConsoleRenderer() as context:
        # print(thread_name, 'inside console reader')

        with context.create_node(thread_name + 'A'):
            # print(thread_name, 'inside Node A')

            inner_fn(context)

            context.create_node(thread_name + 'B')

            with context.create_node(thread_name + 'C'):
                context.create_node(thread_name + 'D')
                # print(thread_name, 'inside Node C')
                context.create_node(thread_name + 'E')

        # print(thread_name, 'outside Node A')

    print(buf.getvalue())


t = [threading.Thread(target=test_multithreading, args=(str(i),)) for i in range(10)]

for thread in t:
    thread.start()



from src.treact.renderers.html import HtmlRenderer


def component(name, c):
    with c.pre:
        context.text_node('#' * (len(name) + 6))
        context.text_node(f"# {name} #")
        context.text_node('#' * (len(name) + 6))


def container(c):
    return c.div(style="border: 1px solid black")


styles = """
button {
    background-color: red;
}
"""


def ui(c):
    with c.html:
        with c.head:
            with c.title:
                c.text_node("Hello world")

            with c.style:
                c.text_node(styles)

        with c.body:
            with c.button(onclick="alert('hello')"):
                c.text_node("Click me")

            with c.ol:
                for i in range(3):
                    with c.li:
                        c.text_node(f"list item {i}")

            component("hello components 👋", c)

            with container(c):
                component("look ma, i m trapped in a container", c)


with HtmlRenderer() as context:
    ui(context)



import pprint
from contextlib import contextmanager

from src.treact.core import NodeContext


@contextmanager
def ConsoleRenderer(buf=None):
    context = NodeContext()
    yield context
    pprint.pprint(context.root, indent=4, stream=buf)



from contextlib import contextmanager

from rich import print
from rich.tree import Tree

from src.treact.core import NodeContext


def render_tree(node):
    if not node:
        return
    t = Tree(node.data)
    for child in node.children:
        r = render_tree(child)
        if r:
            t.add(r)
    return t


@contextmanager
def TreeRenderer(buf=None):
    context = NodeContext()
    yield context
    print(render_tree(context.root), file=buf)



from contextlib import contextmanager
from dataclasses import dataclass
from typing import Optional
from xml.dom.minidom import Document, Node as DomNode

from src.treact.core import Node, NodeContext


@dataclass
class HtmlElement:
    tag: str
    attrs: dict[str, str]


class HtmlNode(Node):
    def __call__(self, *args, **kwargs):
        self.data.attrs.update(kwargs)
        return self


class HtmlNodeContext(NodeContext):
    def create_node(self, data):
        return HtmlNode(data, self)

    def html_node(self, tag: str, attrs: dict[str, str] | None = None):
        if attrs is None:
            attrs = {}
        return self.create_node(HtmlElement(tag, attrs))

    def text_node(self, text: str):
        return self.create_node(text)

    def __getattr__(self, tag_name: str):
        return self.html_node(tag_name)


def render_html(node: Optional[Node], document=Document()) -> Optional[DomNode]:
    if not node:
        return

    if isinstance(node.data, str):
        return document.createTextNode(node.data)

    tag_name = node.data.tag

    element = document.createElement(tag_name)
    for key, value in node.data.attrs.items():
        element.setAttribute(key, value)
    for child in node.children:
        element.appendChild(render_html(child, document))
    return element


@contextmanager
def HtmlRenderer(buf=None):
    context = HtmlNodeContext()
    yield context
    print('<!DOCTYPE html>', file=buf)
    print(render_html(context.root).toprettyxml(), file=buf)



import logging
from dataclasses import dataclass, field
from typing import Optional

logger = logging.getLogger(__name__)


class Node:
    def __init__(self, data, context):
        logger.debug("creating Node %s", data)
        self.data = data
        self.context = context
        self.children = []
        self.depth = 0
        self.hook_self_to_parent()

    def hook_self_to_parent(self):
        hierarchy = self.context.hierarchy
        if hierarchy:
            parent = hierarchy[-1]
            parent.add_child(self)

    def add_child(self, child):
        child.depth = self.depth + 1
        self.children.append(child)

    def __enter__(self):
        logging.debug("entering %s", self.data)
        self.context.enter(self)

    def __exit__(self, exc_type, exc_value, trace):
        logging.debug("exiting %s", self.data)
        self.context.exit(self)

    def __repr__(self):
        data = self.data
        children = self.children
        return f"Node({data=},{children=})"


@dataclass
class NodeContext:
    hierarchy: list[Node] = field(default_factory=list)
    root: Optional[Node] = None

    def create_node(self, data):
        return Node(data, self)

    def enter(self, node: Node):
        self.hierarchy.append(node)

    def exit(self, node: Node):
        if self.hierarchy:
            self.hierarchy.pop()
        self.root = node



from src.treact.renderers.console import ConsoleRenderer


def component(name, context):
    with context.create_node(f"My comp {name}"):
        context.create_node("Text")


def container_wrapper(name, context):
    n = context.create_node(f"My wrapper comp {name}")
    with n:
        component(name, context)
    return n


def tree(context):
    with context.create_node("Root"):
        with context.create_node("A"):
            component("123", context)
            context.create_node("B")
            with context.create_node("C"):
                context.create_node("D")
        context.create_node("E")
        component("456", context)
        for i in range(3):
            context.create_node(f"iter {i}")
        with context.create_node("F"):
            context.create_node("G")
        with container_wrapper("test", context):
            context.create_node("under test")


with ConsoleRenderer() as context:
    tree(context)
